package handler

import (
	"encoding/json"
	"go.opentelemetry.io/contrib/instrumentation/net/http/otelhttp"
	"model/internal/auth"
	"net/http"

	oapimiddleware "github.com/go-openapi/runtime/middleware"
	middleware "github.com/oapi-codegen/nethttp-middleware"

	"model/internal/productassistant"
)

func AppHandler(srv *productassistant.Server, jwtSecret string) http.Handler {
	// Загружаем спецификацию Swagger
	swagger, err := productassistant.GetSwagger()
	if err != nil {
		panic("failed to load swagger spec: " + err.Error())
	}
	// Используем сгенерированный Handler, который принимает StrictServerInterface.
	// Наш Server реализует StrictServerInterface.
	strictHandler := productassistant.NewStrictHandler(srv, nil)

	// Создаем главный обработчик с валидацией запросов по спецификации.
	mainHandler := middleware.OapiRequestValidator(swagger)(productassistant.Handler(strictHandler))

	mux := http.NewServeMux()
	// Маршрут для swagger.json
	mux.HandleFunc("/swagger/swagger.json", func(w http.ResponseWriter, r *http.Request) {
		_ = json.NewEncoder(w).Encode(swagger)
	})
	// Маршрут для Swagger UI
	mux.Handle("/swagger/", oapimiddleware.SwaggerUI(oapimiddleware.SwaggerUIOpts{
		Path:    "/swagger/",
		SpecURL: "/swagger/swagger.json",
	}, nil))

	// Для защищённых маршрутов (например, /favourites/*) применяем JWT middleware.
	protectedMux := http.NewServeMux()
	protectedMux.Handle("/", auth.JWTAuthMiddleware(jwtSecret)(mainHandler))
	mux.Handle("/favourites/", protectedMux)

	mux.Handle("/", middleware.OapiRequestValidator(swagger)(productassistant.Handler(strictHandler)))
	return otelhttp.NewHandler(mux, "api-server")
}
