package kafka

import (
	"context"
	"log"

	"github.com/segmentio/kafka-go"
)

// consumerWrapper реализует интерфейс Consumer.
type consumerWrapper struct {
	reader *kafka.Reader
}

func (c *consumerWrapper) Fetch(ctx context.Context) (Message, error) {
	return c.reader.FetchMessage(ctx)
}

func (c *consumerWrapper) Commit(ctx context.Context, messages ...Message) error {
	return c.reader.CommitMessages(ctx, messages...)
}

func (c *consumerWrapper) Close() error {
	return c.reader.Close()
}

func (c *consumerWrapper) Start(ctx context.Context, handler MessageHandler) {
	go func() {
		for {
			msg, err := c.reader.FetchMessage(ctx)
			if err != nil {
				log.Printf("Error fetching message: %v", err)
				continue
			}
			if err := handler(ctx, msg); err != nil {
				log.Printf("Error handling message: %v", err)
				// Если произошла ошибка, не подтверждаем сообщение.
				continue
			}
			if err := c.reader.CommitMessages(ctx, msg); err != nil {
				log.Printf("Error committing message: %v", err)
			}
		}
	}()
}
