package ml

import (
	"context"

	"github.com/tmc/langchaingo/llms"
	"github.com/tmc/langchaingo/llms/mistral"
)

// LangchangoClient – интерфейс клиента LLM, реализованный с использованием langchaingo.
type LangchangoClient interface {
	GenerateContent(ctx context.Context, prompt string, userQuery string) (string, error)
}

type langchangoClient struct {
	model *mistral.Model
}

func NewLangchangoClient(modelName, apiKey string) (LangchangoClient, error) {
	client, err := mistral.New(
		mistral.WithModel(modelName),
		mistral.WithAPIKey(apiKey),
	)
	if err != nil {
		return nil, err
	}
	return &langchangoClient{model: client}, nil
}

func (c *langchangoClient) GenerateContent(ctx context.Context, prompt string, userQuery string) (string, error) {
	messageHistory := []llms.MessageContent{
		llms.TextParts(llms.ChatMessageTypeSystem, prompt),
		llms.TextParts(llms.ChatMessageTypeHuman, userQuery),
	}
	response, err := c.model.GenerateContent(ctx, messageHistory, llms.WithTemperature(0))
	if err != nil {
		return "", err
	}
	return response.Choices[0].Content, nil
}
