package service

import (
	"context"
	"model/config"

	"model/internal/domain"
	"model/internal/repository"
)

type FavouriteService interface {
	AddToFavourites(ctx context.Context, userID, productID string) error
	DeleteFromFavourites(ctx context.Context, userID, productID string) error
	GetFavouriteProduct(ctx context.Context, userID, productID string) (*domain.AbstractProduct, error)
	ListFavourites(ctx context.Context, userID string) ([]domain.AbstractProduct, error)
}

type favouriteService struct {
	repo repository.StoreRepository
}

func NewFavouriteServiceWithRepo(repo repository.StoreRepository) FavouriteService {
	return &favouriteService{repo: repo}
}

func NewFavouriteService(ctx context.Context, cfg *config.Config) (FavouriteService, error) {
	repo, err := repository.NewStoreRepository(cfg)
	if err != nil {
		return nil, err
	}
	return &favouriteService{repo: repo}, nil
}

func (s *favouriteService) AddToFavourites(ctx context.Context, userID, productID string) error {
	fav := domain.Favourite{
		UserID:    userID,
		ProductID: productID,
	}
	return s.repo.AddFavourite(ctx, fav)
}

func (s *favouriteService) DeleteFromFavourites(ctx context.Context, userID, productID string) error {
	return s.repo.DeleteFavourite(ctx, userID, productID)
}

func (s *favouriteService) GetFavouriteProduct(ctx context.Context, userID, productID string) (*domain.AbstractProduct, error) {
	return s.repo.GetFavouriteProduct(ctx, userID, productID)
}

func (s *favouriteService) ListFavourites(ctx context.Context, userID string) ([]domain.AbstractProduct, error) {
	return s.repo.ListFavourites(ctx, userID)
}
