package service

import (
	"context"
	"model/config"

	"model/internal/domain"
	"model/internal/repository"
)

type ProductService interface {
	GetProductByID(ctx context.Context, id string) (*domain.AbstractProduct, error)
	GetProductsByQuery(ctx context.Context, query string, limit int) ([]domain.AbstractProduct, error)
	GetProductsByIDs(ctx context.Context, ids []string) ([]domain.AbstractProduct, error)
	GetParsedProductsByAbstractID(ctx context.Context, abstractID string) ([]domain.ParsedProduct, error)
}

type productService struct {
	repo repository.StoreRepository
}

func NewProductService(ctx context.Context, cfg *config.Config) (ProductService, error) {
	repo, err := repository.NewStoreRepository(cfg)
	if err != nil {
		return nil, err
	}
	return &productService{repo: repo}, nil
}

func (s *productService) GetProductByID(ctx context.Context, id string) (*domain.AbstractProduct, error) {
	return s.repo.GetProductByID(ctx, id)
}

func (s *productService) GetProductsByQuery(ctx context.Context, query string, limit int) ([]domain.AbstractProduct, error) {
	return s.repo.GetProductsByQuery(ctx, query, limit)
}

func (s *productService) GetProductsByIDs(ctx context.Context, ids []string) ([]domain.AbstractProduct, error) {
	return s.repo.GetProductsByIDs(ctx, ids)
}

// GetParsedProductsByAbstractID возвращает список конкретных (parsed) товаров для абстрактного товара.
func (s *productService) GetParsedProductsByAbstractID(ctx context.Context, abstractID string) ([]domain.ParsedProduct, error) {
	return s.repo.GetParsedProductsByAbstractID(ctx, abstractID)
}
