package service

import (
	"fmt"
	"io"
	"net/http"
	"net/url"
)

// TelegramNotifier – интерфейс для отправки уведомлений через Telegram.
type TelegramNotifier interface {
	SendNotification(userID string, message string) error
}

// TelegramNotifierImpl – реализация, которая отправляет сообщение через Telegram Bot API.
type TelegramNotifierImpl struct {
	botToken string
}

// NewTelegramNotifier возвращает нового TelegramNotifierImpl.
func NewTelegramNotifier(botToken string) TelegramNotifier {
	return &TelegramNotifierImpl{botToken: botToken}
}

func (t *TelegramNotifierImpl) SendNotification(userID string, message string) error {
	apiURL := fmt.Sprintf("https://api.telegram.org/bot%s/sendMessage", t.botToken)
	values := url.Values{}
	values.Set("chat_id", userID)
	values.Set("text", message)

	resp, err := http.PostForm(apiURL, values)
	if err != nil {
		return fmt.Errorf("failed to send telegram message: %w", err)
	}
	defer resp.Body.Close()
	body, _ := io.ReadAll(resp.Body)
	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("telegram API error: %s", string(body))
	}
	return nil
}
